"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getArgs = void 0;
const tslib_1 = require("tslib");
const prompts_1 = require("@clack/prompts");
const commander_1 = require("commander");
const get_anchor_templates_1 = require("./get-anchor-templates");
const get_app_info_1 = require("./get-app-info");
const get_presets_1 = require("./get-presets");
const get_prompts_1 = require("./get-prompts");
function getArgs(argv_1) {
    return tslib_1.__awaiter(this, arguments, void 0, function* (argv, pm = 'npm') {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        // Get app info from package.json
        const app = (0, get_app_info_1.getAppInfo)();
        // Get info about the presets
        const { presets, presetValues } = (0, get_presets_1.getPresets)(app.version);
        const { anchorTemplates, anchorTemplateValues } = (0, get_anchor_templates_1.getAnchorTemplates)();
        // Get the result from the command line
        const input = commander_1.program
            .name(app.name)
            .version(app.version, '-V, --version', help('Output the version number\n'))
            .argument('[name]', 'Name of the project (default: <prompt>)')
            .option('-p, --preset <preset>', help(`Preset to use (default: <prompt>, options: ${presetValues.join(', ')})\n`), (value) => {
            if (!presetValues.includes(value)) {
                throw new Error(`Invalid preset: ${value}`);
            }
            return value;
        })
            .option('--ui <ui-library>', help(`UI library to use (default: <prompt>)\n`))
            .option('-a, --anchor <template>', help(`Name of the Anchor template to use (default: <prompt>, set to "none" to prevent adding Anchor)\n`), (value) => {
            if (!anchorTemplateValues.includes(value)) {
                throw new Error(`Invalid anchor template: ${value}`);
            }
            return value;
        })
            .option('--anchor-build', help(`Build the anchor project`), false)
            .option('--anchor-name <name>', help(`Anchor project name (default: anchor)\n`))
            .option('--anchor-program <name>', help(`Anchor program name\n`))
            .option('--web-name <name>', help(`Web project name (default: web)\n`))
            .option('--web-port <port>', help(`Web project port (default: 3000)\n`))
            .option('-pm, --package-manager <package-manager>', help(`Package manager to use (default: npm)\n`))
            .option('--yarn', help(`Use yarn as the package manager`), false)
            .option('--pnpm', help(`Use pnpm as the package manager`), false)
            .option('-d, --dry-run', help('Dry run (default: false)\n'))
            .helpOption('-h, --help', help('Display help for command\n'))
            .addHelpText('after', `
Examples:
  $ ${app.name} my-app --preset react
  $ ${app.name} my-app --preset react --package-manager yarn
  $ ${app.name} my-app --preset react --anchor basic
      `)
            .parse(argv);
        // Get the optional name argument (positional)
        const name = input.args[0];
        // Get the options from the command line
        const result = input.opts();
        let packageManager = (_a = result.packageManager) !== null && _a !== void 0 ? _a : pm;
        // The 'yarn' and 'pnpm' options are mutually exclusive, and will override the 'packageManager' option
        if (result.pnpm && result.yarn) {
            prompts_1.log.error(`Both pnpm and yarn were specified. Please specify only one.`);
            throw new Error(`Both pnpm and yarn were specified. Please specify only one.`);
        }
        if (result.pnpm) {
            packageManager = 'pnpm';
        }
        if (result.yarn) {
            packageManager = 'yarn';
        }
        // Take the result from the command line and use it to populate the options
        const options = {
            anchor: result.anchor,
            anchorBuild: result.anchorBuild,
            anchorName: (_b = result.anchorName) !== null && _b !== void 0 ? _b : 'anchor',
            anchorProgram: (_d = (_c = result.anchorProgram) !== null && _c !== void 0 ? _c : name) !== null && _d !== void 0 ? _d : '',
            dryRun: (_e = result.dryRun) !== null && _e !== void 0 ? _e : false,
            name: name !== null && name !== void 0 ? name : '',
            package: '',
            packageManager,
            pnpm: result.pnpm ? result.pnpm : false,
            preset: result.preset,
            ui: result.ui,
            yarn: result.yarn ? result.yarn : false,
            webName: (_f = result.webName) !== null && _f !== void 0 ? _f : 'web',
            webPort: (_g = result.webPort) !== null && _g !== void 0 ? _g : 3000,
        };
        // Display the intro
        (0, prompts_1.intro)(`${app.name} ${app.version}`);
        // Get the prompts for any missing options
        const prompts = yield (0, get_prompts_1.getPrompts)({ anchorTemplates, options, presets });
        // Populate the options with the prompts
        if (prompts.name) {
            options.name = prompts.name;
        }
        if (prompts.preset) {
            options.preset = prompts.preset;
        }
        if (prompts.ui) {
            options.ui = prompts.ui;
        }
        if (prompts.anchor) {
            options.anchor = prompts.anchor;
        }
        if (options.anchor !== 'none') {
            options.anchorProgram = prompts.anchorProgram;
        }
        // Validate the options
        if (options.preset && !presetValues.includes(options.preset)) {
            prompts_1.log.error(`Invalid preset: ${options.preset}`);
            throw new Error(`Invalid preset: ${options.preset}`);
        }
        if (!anchorTemplateValues.includes((_h = options.anchor) !== null && _h !== void 0 ? _h : '')) {
            prompts_1.log.error(`Invalid anchor template: ${options.anchor}`);
            throw new Error(`Invalid anchor template: ${options.anchor}`);
        }
        const libs = (0, get_prompts_1.getUiLibraries)(options.preset);
        if (options.ui && !libs.map((l) => l.value).includes(options.ui)) {
            prompts_1.log.error(`Invalid ui library for preset ${options.preset}: ${options.ui}`);
            throw new Error(`Invalid ui library for preset ${options.preset}: ${options.ui}`);
        }
        // Create the package string
        // TODO: In the future, we should allow the user to specify the package and version using the preset option
        if (options.package === '') {
            const packageName = (_j = presets.find((preset) => preset.value === options.preset)) === null || _j === void 0 ? void 0 : _j.package;
            const packageVersion = (_k = presets.find((preset) => preset.value === options.preset)) === null || _k === void 0 ? void 0 : _k.version;
            options.package = `${packageName}@${packageVersion}`;
        }
        return options;
    });
}
exports.getArgs = getArgs;
// Helper function to add a newline before the text
function help(text) {
    return `

  ${text}`;
}
//# sourceMappingURL=get-args.js.map